/*
  Copyright 2004, 2005 Jean-Baptiste Note

  This file is part of prism54usb.

  prism54usb is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  prism54usb is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with prism54usb; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
*/

#ifndef _HAVE_ISL_SM_H_
#define _HAVE_ISL_SM_H_

#include <linux/netdevice.h>
#include <linux/wireless.h>

#ifdef MADWIFI
#include <linux/version.h>
#include "net80211/if_media.h"
#include "net80211/ieee80211_var.h"
#endif /* MADWIFI */

#include "islsm_protocol.h"

/* debug macros */

extern int sm_debug;

#define CONFIG_ISLSM_DEBUG 
#ifdef CONFIG_ISLSM_DEBUG
#define islsm_dbg(format, arg...) if(sm_debug > 1) printk(format "\n" ,##arg)
#else
#define islsm_dbg(format, arg...)
#endif /* CONFIG_ISLSM_DEBUG */
#define islsm_info(format, arg...) if(sm_debug) printk(KERN_INFO __FILE__ ": " format "\n" , ##arg)
#define islsm_warn(format, arg...) printk(KERN_WARN __FILE__ ": " format "\n" , ##arg)
#define islsm_err(format, arg...) printk(KERN_ERR __FILE__ ": " format "\n" ,##arg)

#define islsm_time(arg...) 0LL

//#define islsm_time(arg...) (get_jiffies_64(##arg) - p54u->time)

enum islsm_versions {
	ISLSM_DEVICE_USB_VER1 = 0,
	ISLSM_DEVICE_USB_VER2,
	ISLSM_DEVICE_PCI_VER1,
};

#define ISLSM_TX_TIMEOUT		(HZ/10)

/* softmac management for isl 802.11g devices. For now this softmac
 * relies on the madwifi stack. Plan is to port is to the hostap API.
 * If you would like to do this, please contact me */

// assuming those values are common
// if not, dispatch and add a field in islsm
// This equals 512 * 8 bits
#define ISLSM_MAX_FRAME_SIZE	4096
// we need 3 bytes more for alignment purposes
#define ISLSM_ALIGN_SPACE       3
#define ISLSM_INIT_BLOCK	0x03fc
#define ISLSM_MGMT_MEM_SIZE     0x2000

struct islsm {
	/* state variables */
	/* lmac queues array */
	struct tx_queue         tx_queues[ISLSM_NR_TX_QUEUES];
	int                     err;

	/* reserve this much space ahead of the skb for the needs of the
	   hardware tx queues */
	size_t                  device_tx_header_space;
	/* sm_tx_header_space to be added */
	/* sm_rx_header_space to be added (for monitor mode) */

	enum islsm_versions     device_version;
	/* signalled when particular states in the configuration have
	   been reached : init interrupt, then mgmt readback end */
	struct completion       dev_init_comp;
	int                     wait_flag;

	/* lmac parameters */
	/* base lmac address of the config queue */
	u32                     config_reg;
	/* base lmac address of the data queue */
	u32                     datatx_reg;

	/* number of rates defined in the readback */
	u8                      option4_nelems;

	u8                      *mgmt_frame;
#ifdef MADWIFI
	struct islsm_freq_struct1       finfo3[IEEE80211_CHAN_MAX];
	struct islsm_freq_substruct2    finfo4[IEEE80211_CHAN_MAX][ISLSM_CHANNEL_DEFS];
	struct islsm_freq_struct3       finfo6[IEEE80211_CHAN_MAX];
#else /* MADWIFI */
	struct islsm_freq_struct1       finfo3[ISLSM_NR_CHANNELS+1];
	struct islsm_freq_substruct2    finfo4[ISLSM_NR_CHANNELS+1][ISLSM_CHANNEL_DEFS];
	struct islsm_freq_struct3       finfo6[ISLSM_NR_CHANNELS+1];
#endif /* MADWIFI */

#ifdef MADWIFI
	struct ieee80211com     sc_ic;
	struct ieee80211_beacon_offsets	islsm_boff;
	int			(*ieee80211_newstate)(struct ieee80211com *,
						      enum ieee80211_state, int);
	struct timer_list	scan_ch;	/* AP scan timer */
#else
	/* we need to maintain wireless mode */
	int                     iw_mode;
#endif /* MADWIFI */

//	struct timer_list       stats_update;
	struct net_device_stats statistics;
#ifdef CONFIG_NET_WIRELESS
	struct iw_statistics    iwstatistics;
#endif /* CONFIG_NET_WIRELESS */

	/* device-specific callbacks.
	   We handle three types of devices : 
	   - usb first generation
	   - usb second generation
	   - pci devices 
	*/

	/* boots the device */
	int (*isl_boot)(struct net_device *);
	int (*isl_stop)(struct net_device *);
	/* transmits the data in the skb to the lmac */
	int (*isl_tx)(struct net_device *, struct sk_buff *, u32);

	// this is filled with p54u or islpci device reference
	void *hardware;
};

extern int islsm_wait_timeout(struct islsm *islsm, unsigned int delay);
extern int islsm_setup_net(struct net_device *netdev);
extern int islsm_unsetup_net(struct net_device *netdev);

#endif /* _HAVE_ISL_SM_H_ */
