/*
  Copyright 2004, 2005 Jean-Baptiste Note

  This file is part of prism54usb.

  prism54usb is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  prism54usb is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with prism54usb; if not, write to the Free Software Foundation,
  Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

#include <linux/netdevice.h>
#include <linux/skbuff.h>
#include "islsm_protocol.h"
#include "isl_sm.h"

/*
 * Second reorganization : request = queue + lmac + skb
 */

static
void islsm_make_prism_header_skb(struct sk_buff* skb, u16 type, u32 req_id, u16 reqtype, u16 flags)
{
	struct islsm_tx_control *islsm_txc;
	int len = skb->len;
	BUG_ON(skb_headroom(skb) < SIZE_TX_CONTROL);
	islsm_txc = (struct islsm_tx_control *) skb_push(skb,SIZE_TX_CONTROL);
	islsm_txc->id.magic1  = cpu_to_le16(type);
	islsm_txc->id.length = cpu_to_le16(len);
	islsm_txc->req_id = req_id;
	islsm_txc->type = cpu_to_le16(reqtype);
	islsm_txc->flags = cpu_to_le16(flags);

	if ( ((long)skb->data & 0x03) )
	{
		islsm_dbg("skb->data is not 32-bits aligned !");
		BUG();
	}
	// should do the tx here maybe ?
}


/* TX/RX/MODE packet */
static
int islsm_make_rx_filter_skb(struct net_device *netdev, struct sk_buff *skb,
			    u16 filter_type, u8 *bssid,
			    u16 magic2, u32 magic3, u32 magic4, 
			    u16 magic8, u16 magic9)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct islsm_tx_control_filter *txcf;
	unsigned int i ;
	
	/* size should be okay */
	BUG_ON(skb_headroom(skb) < SIZE_TX_CONTROL_FILTER);
	txcf = (struct islsm_tx_control_filter *) skb_push(skb,SIZE_TX_CONTROL_FILTER);	
	memset(txcf,0,sizeof(*txcf));
	
	txcf->filter_type = cpu_to_le16(filter_type);

	txcf->magic2 = cpu_to_le16(magic2);
	txcf->magic3 = cpu_to_le32(magic3);
	txcf->magic4 = cpu_to_le32(magic4);
	
	if (islsm->device_version == ISLSM_DEVICE_USB_VER1)
	{
		txcf->magic6 = cpu_to_le32(ISLSM_TX_CONTROL_FILTER_MAGIC6_VER1);
		txcf->magic7 = cpu_to_le32(ISLSM_TX_CONTROL_FILTER_MAGIC7_VER1);
	}
	else if (islsm->device_version == ISLSM_DEVICE_USB_VER2) {
		txcf->magic6 = cpu_to_le32(ISLSM_TX_CONTROL_FILTER_MAGIC6_VER2);
		txcf->magic7 = cpu_to_le32(ISLSM_TX_CONTROL_FILTER_MAGIC7_VER2);
	}
	else {
		/* PCI */
		txcf->magic6 = cpu_to_le32(ISLSM_TX_CONTROL_FILTER_MAGIC6_PCI);
		txcf->magic7 = cpu_to_le32(ISLSM_TX_CONTROL_FILTER_MAGIC7_PCI);
	}
	txcf->magic8 = cpu_to_le16(magic8);
	txcf->magic9 = cpu_to_le16(magic9);

 	/* my mac address  */
 	memcpy(&txcf->destination[0],netdev->dev_addr,ETH_ALEN);

	/* the other mac address */
	for( i = 0 ; i < ETH_ALEN ; i++)
		txcf->source[i] = bssid[i];
	
	/* what's next */
	islsm_make_prism_header_skb(skb,ISLSM_TX_CONTROL_NORESPONSE_ID,0,ISLSM_TX_CONTROL_TYPE_FILTER_SET,0);
	return islsm->isl_tx(netdev,skb,ISLSM_TX_QUEUE_CONFIG);
}


/* Frequency change packet building */
/* common filling */
static inline void
fill_basic_channel(struct islsm *islsm, 
		   struct islsm_tx_control_channel *tx_chan,
		   unsigned int chan, unsigned int freq) 
{	
	/* we normalized this */
	unsigned int array_index = chan;

 	if (islsm->finfo6[array_index].freq != 0)
	{
		/* the information was present */
		tx_chan->freq = islsm->finfo6[array_index];
	}
	else
	{
		unsigned int i;
		/* fill with observed default value */
		islsm_info("channel of which only partial information exists on third array");	
/*
  we could use the [0] as a repository for default values. for now
  tx_chan->freq = islsm->finfo6[0];
*/
#define L3_DEFAULT_VAL 0x8000U
		for(i = 0 ; i < 4 ; i++)
			tx_chan->freq.data[i]=cpu_to_le16(L3_DEFAULT_VAL);
		tx_chan->freq.freq = cpu_to_le16(freq);
	}
	
        /* This part remains to be understood ; i'm sure there are
	   better approximations  */
	tx_chan->nsettings = islsm->option4_nelems;
	// Soon, soon, this will be fully understood and gotten rid of
 	tx_chan->unknown1 = 0x48;
	memcpy(tx_chan->l1data, islsm->finfo3[array_index].data,sizeof(islsm->finfo3[array_index].data));
 	tx_chan->l1data[3] = islsm->finfo3[array_index].data[3];
#undef L3_DEFAULT_VAL

	memcpy(tx_chan->str,&islsm->finfo4[array_index],sizeof(islsm->finfo4[array_index]));
}

/* chan is the ieee channel number */
static
int islsm_make_tx_control_channel_skb(struct net_device *netdev, struct sk_buff *skb,
				     unsigned int chan, unsigned int freq,
				     u16 magic1, u16 magic2)
{
	struct islsm *islsm = netdev_priv(netdev);
 	struct islsm_tx_control_channel *tx_chan = 0;
	size_t mysize;

	mysize = islsm->device_version == ISLSM_DEVICE_USB_VER1 ? SIZE_TX_CONTROL_CHANNEL : SIZE_TX_CONTROL_CHANNEL_VER2;

	BUG_ON(skb_headroom(skb) < mysize);
	tx_chan = (struct islsm_tx_control_channel *) skb_push(skb,mysize);

	memset(tx_chan,0,sizeof(*tx_chan));

	tx_chan->magic1 = cpu_to_le16(magic1);
	tx_chan->magic2 = cpu_to_le16(magic2);
	/* keep this here for now */
	if (chan == 14 && magic1 == ISLSM_TX_CONTROL_CHANNEL_MAGIC1_SCAN)
		tx_chan->magic1 |= cpu_to_le16(ISLSM_TX_CONTROL_CHANNEL_MAGIC1_SCAN_WRAP);

	fill_basic_channel(islsm,tx_chan,chan,freq);

	/* version 2 devices. This is here to emphasize the difference.
	   Of course it sits better in fill_basic_channel */
	if (islsm->device_version != ISLSM_DEVICE_USB_VER1)
	{
		struct islsm_tx_control_channel_ver2 *tx_chan2 = 
			(struct islsm_tx_control_channel_ver2 *) tx_chan; /* only 4 bytes longer */

		/* check that this is indeed it ! */
		/* Why did they change this ? */
		memcpy(&(tx_chan2->basic_packet.padding),
		       tx_chan2->basic_packet.l1data,
		       sizeof(tx_chan2->basic_packet.l1data));
		tx_chan2->padding = 0;
	}

	islsm_make_prism_header_skb(skb,ISLSM_TX_CONTROL_NORESPONSE_ID,0,ISLSM_TX_CONTROL_TYPE_CHANNEL_CHANGE,0);
	return islsm->isl_tx(netdev,skb,islsm->config_reg);
}

static
int islsm_make_control_led_skb(struct net_device *netdev, struct sk_buff *skb,
			      u16 mode, u16 perm_setting, u16 temp_setting, u16 duration)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct islsm_tx_control_led *payload;

	BUG_ON(skb_headroom(skb) < SIZE_TX_CONTROL_LED);
	payload = (struct islsm_tx_control_led *) skb_push(skb,SIZE_TX_CONTROL_LED);
	
	payload->mode = cpu_to_le16(mode);
	payload->led_setting_temporary = cpu_to_le16(temp_setting);
	payload->led_setting_permanent = cpu_to_le16(perm_setting);
	payload->temporary_setting_duration = cpu_to_le16(duration);

	islsm_make_prism_header_skb(skb, ISLSM_TX_CONTROL_NORESPONSE_ID, 0, ISLSM_TX_CONTROL_TYPE_LED,0);
	return islsm->isl_tx(netdev,skb, islsm->config_reg);
}

static
int islsm_make_freequeue_skb(struct net_device *netdev, struct sk_buff* skb,
			    u8 queue_id)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct islsm_tx_control_freequeue *payload;
	
	BUG_ON(skb_headroom(skb) < SIZE_TX_CONTROL_FREEQUEUE);
	payload = (struct islsm_tx_control_freequeue *) skb_push(skb,SIZE_TX_CONTROL_FREEQUEUE);
	payload->queue = cpu_to_le32(ISLSM_TX_QUEUE_TX(queue_id));

	islsm_make_prism_header_skb(skb, ISLSM_TX_CONTROL_NORESPONSE_ID, 0, ISLSM_TX_CONTROL_TYPE_FREEQUEUE,0);
	return islsm->isl_tx(netdev, skb, islsm->config_reg);
}

static
int islsm_make_readback_skb(struct net_device *netdev, struct sk_buff* skb,
			   unsigned offset, unsigned length)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct islsm_control_eeprom *payload;
	u8 queue_id;
	int err;	

	BUG_ON(skb_headroom(skb) < SIZE_CONTROL_EEPROM + length);
	payload = (struct islsm_control_eeprom *) skb_push(skb,SIZE_CONTROL_EEPROM + length);
	
	// no need for this memset
	//memset(payload, 0, SIZE_CONTROL_EEPROM + length);
	payload->offset = cpu_to_le16(offset);
	payload->len = cpu_to_le16(length);

	queue_id = islsm_txqueue_alloc(netdev, skb);
	if (!queue_id)
		return -EIO;

	islsm_make_prism_header_skb(skb, ISLSM_ID_EEPROM_READBACK, queue_id, ISLSM_TX_CONTROL_TYPE_EEPROM_READBACK,0);

	err = islsm->isl_tx(netdev, skb, ISLSM_TX_QUEUE_TX(queue_id));

	if (err)
		islsm_txqueue_free(netdev,queue_id);

	return err;
}

/* send packet, higly suboptimal version, and API will change.  I'd
   really like to be able to ask the upper layers to send a skb with a
   minimal headroom in it... */

static
int islsm_make_tx_datapkt_skb(struct net_device *netdev, struct sk_buff *skb, unsigned short padding,
			     u8 queue_id, 
			     u32 id_magic1, u16 req_type, u16 flags,
			     u32 magic4, u32 magic5,
			     u32 frame_type)
{
	// will need to lookup rate & al
	struct islsm_tx_control_allocdata *txd;
	struct islsm_tx_control *islsm_txc;
	struct islsm *islsm = netdev_priv(netdev);
	size_t datalen = skb->len;

	BUG_ON(skb_headroom(skb) < SIZE_TX_CONTROL_ALLOCDATA + padding);
	txd = (struct islsm_tx_control_allocdata *)skb_push(skb,SIZE_TX_CONTROL_ALLOCDATA + padding);
	memset(txd, 0, SIZE_TX_CONTROL_ALLOCDATA);

	/* this is frail, use a lookup function or pass usb_id as
	   argument. Better still, do the alloc ourselves  */
	u32 identifier = queue_id;

	/* The fact that the length embeds the full header or not is
	 * certainly specified in magic1 (so that we parse with no
	 * backtrack) : find out how. See also examples on input */
	txd->frame_type = cpu_to_le32(frame_type);

	txd->magic4 = cpu_to_le32(magic4);
	txd->magic5 = cpu_to_le32(magic5);

	txd->frame_align[0] = padding;
	
	BUG_ON(skb_headroom(skb) < SIZE_TX_CONTROL);
	islsm_txc = (struct islsm_tx_control *) skb_push(skb,SIZE_TX_CONTROL);
	islsm_txc->id.magic1  = cpu_to_le16(id_magic1);
	islsm_txc->id.length  = cpu_to_le16(datalen);
	islsm_txc->req_id = identifier;
	islsm_txc->type = cpu_to_le16(req_type);
	islsm_txc->flags = cpu_to_le16(flags);
	
	return islsm->isl_tx(netdev, skb, ISLSM_TX_QUEUE_TX(queue_id));
}

/* functions for actual sending */
static inline
struct sk_buff * islsm_skb_alloc(struct islsm *islsm, size_t headroom)
{
	/* FIXME : in monitor mode for rx packets, we should allocate
	 * enough header room for the monitor header, avoiding a
	 * copy */
	/* Check ! : should be atomic allocation in tx path and
	 * non-atomic in rx path */
	struct sk_buff *skb = dev_alloc_skb(ISLSM_MAX_FRAME_SIZE+ISLSM_ALIGN_SPACE);
	headroom += islsm->device_tx_header_space;
	/* realign buffer */
	if (skb) {
		// This naturally makes sure that the beginning of the
		// frame will be 32-bits aligned
		skb_push(skb, skb_headroom(skb));
		skb_trim(skb, 0);
		skb_reserve(skb,headroom);
	}
	else {
		islsm_err("skb allocation failed");
	}
	return skb;
}

static
int islsm_config_readback(struct net_device *netdev, 
			 unsigned offset, unsigned length)
{
	struct sk_buff *mgmt_packet;
	struct islsm *islsm = netdev_priv(netdev);

	mgmt_packet = islsm_skb_alloc(islsm, FULL_CONTROL_EEPROM + length);
  	if (!mgmt_packet)
  	  return -ENOMEM;
	/* TODO :
	 * The pipe used for readback should be dynamically allocated */
	return islsm_make_readback_skb(netdev, mgmt_packet, offset, length);
}


int islsm_set_filter(struct net_device *netdev, 
		    u16 filter_type, u8 *bssid,
		    u16 magic2, u32 magic3, u32 magic4, 
		    u16 magic8, u16 magic9)
{
	struct islsm *islsm = netdev_priv(netdev);
 	struct sk_buff* filter_packet;
 	
 	filter_packet = islsm_skb_alloc(islsm,FULL_TX_CONTROL_FILTER);
  	if (!filter_packet)
  	  return -ENOMEM;
	
	// FIXME : what does an error entail in the madwifi stack ?
 	return islsm_make_rx_filter_skb(netdev, filter_packet, filter_type, bssid, 
				       magic2, magic3, magic4, magic8, magic9);
}

int islsm_stats_readback(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct sk_buff *stats_packet;
	u8 queue_id;
	int err;

	stats_packet = islsm_skb_alloc(islsm, SIZE_TX_CONTROL + ISLSM_SIZE_STATS);
  	if (!stats_packet)
		return -ENOMEM;

	queue_id = islsm_txqueue_alloc(netdev, stats_packet);
	if (!queue_id)
		return -EIO;

	/* leave room for the device to write */
	skb_push(stats_packet, ISLSM_SIZE_STATS);
	islsm_make_prism_header_skb(stats_packet, ISLSM_ID_EEPROM_READBACK, queue_id, ISLSM_TX_CONTROL_TYPE_STAT_READBACK,0);

	err = islsm->isl_tx(netdev, stats_packet, ISLSM_TX_QUEUE_TX(queue_id));

	if (err)
	{
		/* with the skb do what ? */
		islsm_txqueue_free(netdev,queue_id);
	}

	return err;
}

int islsm_freq_change(struct net_device *netdev, 
		     u16 channel, u16 freq, 
		     u16 magic1, u16 magic2)
{
	struct islsm *islsm = netdev_priv(netdev);
 	struct sk_buff * channel_packet;
	size_t mysize;

	mysize = islsm->device_version == ISLSM_DEVICE_USB_VER1 ? FULL_TX_CONTROL_CHANNEL : FULL_TX_CONTROL_CHANNEL_VER2;
  
 	channel_packet = islsm_skb_alloc(islsm,mysize);
 	
 	if (!channel_packet)
		return -ENOMEM;

	return islsm_make_tx_control_channel_skb(netdev, channel_packet, 
						channel, freq,
						magic1, magic2);
}

int islsm_data_tx(struct net_device *netdev, unsigned short padding,
		 u32 id_magic1, u16 req_type, u16 flags,
		 u32 magic4, u32 magic5,
		 u32 frame_type, 
		 struct sk_buff *skb)
{
	struct islsm *islsm = netdev_priv(netdev);
	int err = 0;
	unsigned int queue_id;
	int my_padding = 0;

	/* FIXME : properly handle realignment issues 
	   FIXME : is there a way to ask the upper layers for a minimal
	   headroom ? */

	/* at this step we can also make use of the padding to realign
	   beginning of the packet to a multiple of 32 bits */

	islsm_dbg("data length %i, padding %i",skb->len,padding);	
	my_padding = ( ( ((u32)skb->data) % 4) + 4 - (FULL_TX_CONTROL_ALLOCDATA % 4)) % 4;
	my_padding = (my_padding == 0) ? 4 : my_padding;
	islsm_dbg("my computed padding is %i, real %i",my_padding, padding);	

	/* now use my padding instead of theirs */
	padding = my_padding;

	/* get an isl lmac queue to send it to */
	queue_id = islsm_txqueue_alloc(netdev, skb);
	if (queue_id == 0)
	{
		goto bad_skb;
	}

	if (skb_headroom(skb) < FULL_TX_CONTROL_ALLOCDATA + TX_MAX_PADDING + islsm->device_tx_header_space)
	{
		/* This should not be triggered now that we set hard_header_len */
		islsm_dbg("****** Problem in skb to be tx'd : headroom too small *******");
		err = skb_cow(skb,FULL_TX_CONTROL_ALLOCDATA + padding + islsm->device_tx_header_space);
		if (err)
		{
			islsm_dbg("dropped tx skb do to error on cow");
			goto bad_frq;
		}
	}

	/* If the txqueue if full, then we should return an error, so
	 * that the skb is freed, and free the lmac tx queue */
 	err = islsm_make_tx_datapkt_skb(netdev, skb, padding, queue_id, 
					id_magic1, req_type, flags,
					magic4, magic5,
					frame_type);

	if (err)
	{
		islsm_dbg("Problem with full tx queue");
		goto bad_frq;
	}

	return 0;

 bad_frq:
	islsm_txqueue_free(netdev,queue_id);
 bad_skb:
	dev_kfree_skb(skb);
	/* seems we have no right to return != 0 */
	return 0;
}


int islsm_empty_queues(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct tx_queue *tx_queue_array = &islsm->tx_queues[0];
	unsigned int i;
	islsm_dbg("freeing all lmac queues");

	for(i=0;i<ISLSM_NR_TX_QUEUES;i++) {
		u8 queue_id;
		struct sk_buff *free_packet;
		if (tx_queue_array[i].usb_id)
		{
			islsm_dbg("something left in %i, id %i",i,tx_queue_array[i].usb_id);
			free_packet = islsm_skb_alloc(islsm,FULL_TX_CONTROL_FREEQUEUE);
			queue_id = tx_queue_array[i].id;
			islsm_make_freequeue_skb(netdev, free_packet, queue_id);
		}
	}
	return 0;
}

/* testing function */
int islsm_outofband_msg(struct net_device *netdev,
			void *buf, size_t size, unsigned lmac_addr)
{
	struct islsm *islsm = netdev_priv(netdev);
	struct sk_buff *skb = islsm_skb_alloc(islsm,0);
	int err;
	u32 queue_id = 0;

	if (!skb)
	{
		islsm_err("skb alloc failed for out-of-band message");
		return -ENOMEM;
	}
	memcpy(skb->data, buf, size);
	skb_put(skb,size);

	if (!lmac_addr)
	{
		queue_id = islsm_txqueue_alloc(netdev, skb);
		lmac_addr = ISLSM_TX_QUEUE_TX(queue_id);
	}

	err = islsm->isl_tx(netdev, skb, lmac_addr);

	if (err && queue_id)
		islsm_txqueue_free(netdev, queue_id);

	return err;
}

static void timeout_complete(unsigned long data) {
	struct islsm *islsm = (struct islsm *) data;
	islsm->wait_flag = -ETIMEDOUT;
	islsm_dbg("timeout waiting");
	complete(&islsm->dev_init_comp);
	return;
}

int islsm_wait_timeout(struct islsm *islsm, unsigned int delay)
{
	struct timer_list timer;
	
	init_timer(&timer);
	timer.expires = jiffies + delay;
	timer.data = (unsigned long)(islsm);
	timer.function = timeout_complete;

	islsm->wait_flag = 0;

	add_timer(&timer);
	wait_for_completion(&islsm->dev_init_comp);
	del_timer_sync(&timer);

	return islsm->wait_flag;
}
EXPORT_SYMBOL(islsm_wait_timeout);

int islsm_mgmt_readback(struct net_device *netdev)
{
	struct islsm *islsm = netdev_priv(netdev);
	int i, len;

	islsm_info("%s: Mgmt readback.\n", netdev->name);
	
	/* This allocate 8K, is this huge ? */
	islsm->mgmt_frame = kmalloc(ISLSM_MGMT_MEM_SIZE, GFP_KERNEL);
	if (!islsm->mgmt_frame)
	{
		islsm_dbg("could not allocate mgmt frame space for mgmt readback");
		return -ENOMEM;
	}

	i = ISLSM_MGMT_MEM_SIZE;
	
	/* Each iteration will trigger a pipe response */
	/* We got a problem here, Olivier's computer is too fast ! --
	 * either we wait for the block to be returned, or we allocate
	 * other pipes to make the requests */
	while (i > 0) {
		len = i > ISLSM_INIT_BLOCK ? ISLSM_INIT_BLOCK : i;
		islsm_config_readback(netdev, ISLSM_MGMT_MEM_SIZE - i, len);
		if (islsm->err)
			break;
		/* TODO : (after correct rx queuing mechanism) try to do
		 * those things concurrently by allocating different
		 * lmac addresses */
		/* Wait for the device's response before returning :
		 * we reuse the dev_init_comp semaphore */
		if ((islsm->err = islsm_wait_timeout(islsm,2*HZ)))
			break;

		i -= ISLSM_INIT_BLOCK;
	}
	islsm_info("%s: Mgmt readback returns %i.\n", netdev->name, islsm->err);
	return islsm->err;
}
EXPORT_SYMBOL(islsm_mgmt_readback);
