/*
	rt2x00 SourceForge Project - <http://rt2x00.serialmonkey.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the
	Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	Licensed under the GNU GPL
	Original code supplied under license from RaLink Inc, 2003
 */

/*
	Module Name: rt2x00_adapter.h

	Abstract: Adapter structures and function declarations.

	Revision History:
	Who		When		What
	--------	-----------	-----------------------------
	IvD		<releasedate>	Initial Release.
 */ 

#ifndef __RT2X00_ADAPTER_H
#define __RT2X00_ADAPTER_H

#include "rt2x00_defines.h"
#include "rt2x00_compatibility.h"

/*
 * Driver information.
 */
#define DRV_NAME			"rt2x00"
#define DRV_NAME_PCI			"rt2x00pci"
#define DRV_NAME_USB			"rt2x00usb"
#define DRV_VERSION 			"2.0.0-alpha"
#define DRV_RELDATE 			"Not released"
#define DRV_PROJECT 			"http://rt2x00.serialmonkey.com"

/*
 * Debug defines.
 */
/* Module option parameter */
extern char	enable_debug;
#ifdef CONFIG_RT2X00_DEBUG
#define DEBUG_PRINTK(__message...)	do{ if(enable_debug) printk(__message); }while(0)
#else
#define DEBUG_PRINTK(__message...)	do{  }while(0)
#endif /* CONFIG_RT2X00_DEBUG */

/* Various debug levels. */
#define PANIC(__message...)		printk(KERN_PANIC DRV_NAME ": Panic - " __message)
#define ERROR(__message...)		printk(KERN_ERR DRV_NAME ": Error - " __message)
#define WARNING(__message...)		DEBUG_PRINTK(KERN_WARNING DRV_NAME ": Warning - " __message)
#define NOTICE(__message...)		DEBUG_PRINTK(KERN_NOTICE DRV_NAME ": Notice - " __message)
#define INFO(__message...)		DEBUG_PRINTK(KERN_INFO DRV_NAME ": Info - " __message)
#define DEBUG(__message...)		DEBUG_PRINTK(KERN_DEBUG DRV_NAME ": Debug - " __message)

/* Defines to easily add a mac address to a debug message. */
#define PRINT_ADDR_STRING		"%02x:%02x:%02x:%02x:%02x:%02x"
#define PRINT_ADDR(__bssid)		(__bssid)[0], (__bssid)[1], (__bssid)[2], (__bssid)[3], (__bssid)[4], (__bssid)[5]

#define SET_FLAG(__struct, __flag)	( (__struct)->flags |= (__flag) )
#define CLEAR_FLAG(__struct, __flag)	( (__struct)->flags &= ~(__flag) )
#define CLEAR_FLAGS(__struct)		( (__struct)->flags = 0 )
#define TEST_FLAG(__struct, __flag)	( ((__struct)->flags & (__flag)) != 0 )
#define GET_FLAG(__struct, __flag)	( (__struct)->flags & (__flag) )

/*
 * _rt2x00_adapter flags
 */
#define BIT(__x)			( 0x00000001 << __x )

#define ADAPTER_ENABLED			BIT(0)
#define ADAPTER_INTERRUPT_ENABLED	BIT(1)
#define ADAPTER_RADIO_ON		BIT(2)
#define ADAPTER_SCAN_IN_PROGRESS	BIT(3)

#define COMMIT_PENDING			( BIT(5) | BIT(6) | BIT(7) | BIT(8) | BIT(9) | BIT(10) | BIT(11) )
#define COMMIT_IN_PROGRESS		BIT(4)
#define COMMIT_ESSID			BIT(5)
#define COMMIT_BSSID			BIT(6)
#define COMMIT_NETWORK			BIT(7)
#define COMMIT_MODE			BIT(8)
#define COMMIT_CRYPTO			BIT(9)
#define COMMIT_POWER			BIT(10)
#define COMMIT_CHANNEL			BIT(11)

#define NETWORK_802_11_A		BIT(12)
#define NETWORK_802_11_B		BIT(13)
#define NETWORK_802_11_G		BIT(14)
#define NETWORK_802_11_BG		( BIT(13) | BIT(14) )
#define NETWORK_802_11_ABG		( BIT(12) | BIT(13) | BIT(14) )

#define ENABLED_POWER			BIT(15)
#define ENABLED_SENS			BIT(16)
#define ENABLED_RTS			BIT(17)
#define ENABLED_FRAG			BIT(18)

#define ACCEPT_UNICAST			BIT(19)
#define ACCEPT_MULTICAST		BIT(20)
#define ACCEPT_ALLMULTICAST		BIT(21)
#define ACCEPT_PROMISC			BIT(22)

#define FIX_RATE			BIT(23)
#define FIX_RTS				BIT(24)
#define FIX_FRAG			BIT(25)
#define FIX_TXPOWER			BIT(26)
#define FIX_BSSID			BIT(27)

#define CRYPTO_ENABLED			BIT(28)
#define CRYPTO_RESTRICT			BIT(29)
#define CRYPTO_WEP			BIT(30)
#define CRYPTO_WPA			BIT(31)

/* 
 * Private IOCTL commands
 */
#define SIOCSIWPRIVNETWORK		SIOCIWFIRSTPRIV + 0x0000
#define SIOCGIWPRIVNETWORK		SIOCIWFIRSTPRIV + 0x0001
#define SIOCSIWPRIVPREAMBLE		SIOCIWFIRSTPRIV + 0x0002
#define SIOCGIWPRIVPREAMBLE		SIOCIWFIRSTPRIV + 0x0003
#define SIOCSIWPRIVANTENNA		SIOCIWFIRSTPRIV + 0x0004
#define SIOCGIWPRIVANTENNA		SIOCIWFIRSTPRIV + 0x0005
#define SIOCSIWPRIVWPA			SIOCIWFIRSTPRIV + 0x0006
#define SIOCGIWPRIVWPA			SIOCIWFIRSTPRIV + 0x0007
#define SIOCSIWPRIVMAC			SIOCIWFIRSTPRIV + 0x0008
#define SIOCGIWPRIVMAC			SIOCIWFIRSTPRIV + 0x0009

/*
 * Flags required for private ioctl
 */
#define NETWORK_A			0x01	/* SIOCSIWPRIVNETWORK */
#define NETWORK_B			0x02	/* SIOCSIWPRIVNETWORK */
#define NETWORK_G			0x04	/* SIOCSIWPRIVNETWORK */
#define NETWORK_BG			0x06	/* SIOCSIWPRIVNETWORK */
#define NETWORK_ABG			0x07	/* SIOCSIWPRIVNETWORK */
#define PREAMBLE_LONG			0x01	/* SIOCSIWPRIVPREAMBLE */
#define PREAMBLE_SHORT			0x02	/* SIOCSIWPRIVPREAMBLE */
#define ANTENNA_TX			0x0F	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_TX_DIV			0x01	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_TX_A			0x02	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_TX_B			0x04	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_RX			0xF0	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_RX_DIV			0x10	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_RX_A			0x20	/* SIOCSIWPRIVANTENNA */
#define ANTENNA_RX_B			0x40	/* SIOCSIWPRIVANTENNA */

/*
 * Types for _crypto structure
 */
#define CRYPTO_TYPE_WEP		0x01
#define CRYPTO_TYPE_WPA		0x0c
#define CRYPTO_TYPE_TKIP	0x04
#define CRYPTO_TYPE_AES		0x08

/*
 * The _crypto flags values are equal to the wireless extensions flags in IW_ENCODE_MODE
 */
struct _crypto {
	u16	flags;
	u8	type;
	u8	length;
	char	*key;
} __attribute__ ((packed));

/*
 * The channel capability depends on 2 values, value (channel no.) and freq.
 */
struct _channel {
	u8				value;
	u8				tx_power;
	u16				freq;
	u32				rf;
} __attribute__ ((packed));

struct _rate {
	u8				value;
	u8				plcp;
} __attribute__ ((packed));

/*
 * The _adapter_capability arrays contain all acceptable values for this adapter type.
 * When a value is changed using an ioctl command it will consult these arrays
 * to verify the new value is allowed.
 * When changing the network type, various configuration values should again be
 * checked to see if they are still acceptable. If not, they should be immediatly changed.
 */
struct _rt2x00_capability {
	/*
	 * These capability arrays should be written according
	 * to the specifications of the network type.
	 */
	u8				tx_powers[IW_MAX_TXPOWER];
	struct _rate			bit_rates[MAX_BIT_RATES];
	struct _channel			channels[MAX_FREQUENCIES];

	/*
	 * Maximum basic rate, this differs between 802.11a and 802.11b networks.
	 */
	u8				basic_rate;

	/*
	 * The various sizes of acceptable values in the capability arrays.
	 */
	u8				size_tx_powers;
	u8				size_bit_rates;
	u8				size_channels;
} __attribute__ ((packed));

/*
 * The _power structure is only used in the ioctl funcions, to store the values, and flags temporary.
 * This structure will be stored in the _commit structure.
 * After the data is being commited it will be stored separately.
 */
struct _power {
	u16				flags;
	u16				period;
	u16				timeout;
	u16				__pad;		/* Unused, just for alignment. */
} __attribute__ ((packed));

/*
 * pending configuration commit structure
 * This structure is protected by the config_lock.
 */
struct _config_commit {
	char				essid[IW_ESSID_MAX_SIZE];	/* COMMIT_ESSID */
	char				bssid[ETH_ALEN];		/* COMMIT_BSSID */
	u8				network;			/* COMMIT_NETWORK */
	u8				mode;				/* COMMIT_MODE */
	struct _crypto			crypto;				/* COMMIT_CRYPTO */
	struct _power			power;				/* COMMIT_POWER */
	u16				channel;			/* COMMIT_CHANNEL */
} __attribute__ ((packed));

struct _rt2x00_config {
	/*
	 * When not NULL this points to the pending data that still has to be committed.
	 * It is completely handled by the rt2x00_commit() and rt2x00_ioctl_commit()
	 * functions.
	 */
	struct _config_commit		*commits;

	char				essid[IW_ESSID_MAX_SIZE];
	char				nickn[IW_ESSID_MAX_SIZE];
	char				bssid[ETH_ALEN];
	/*
	 * Connection mode, i.e. IW_MODE_ADHOC, IW_MODE_INFRA etc...
	 * The variable config_mode indicates which mode is configured.
	 * But certain modes IW_MODE_AUTO and IW_MODE_SECOND may decide
	 * to start a connection in a different mode.
	 * For this is the active_mode variable, which is the mode the
	 * adapter is currently in. When not connected this variable is set to 0,
	 * while config_mode must remain intact.
	 */
	u8				config_mode;
	u8				active_mode;

	s8				rssi;
	u8				preamble;	/* Preamble duration (us). */
	u8				antenna;
	u8				tx_power;	/* Index number of adapter->capability->tx_powers[]. */

	u8				max_rate;	/* Index number of adapter->capability->bit_rates[]. */
	u8				cur_rate;	/* Index number of adapter->capability->bit_rates[]. */
	u8				short_retry;
	u8				long_retry;

	u8				channel;	/* Index number of adapter->capability->channels[]. */
	u8				__pad;		/* Unused, just for alignment. */
	s16				sensitivity;


	u16				fragmentation;
	u16				rts_treshold;

	/*
	 * These values are durations in us.
	 * They are used to calculate packet duraton, pifs eifs etc...
	 */
	u8				beacon;
	u8				plcp;
	u8				sifs;
	u8				slot_time;


	/*
	 * Power management variables.
	 */
	u8				atim_count;
	u8				cfp_count;
	u16				atim_period;
	u16				cfp_period;

	/*
	 * The sequence number for outgoing packets is stored here.
	 * After each send packet this must be incremented using the SEQUENCE_INC() define.
	 * This define makes the proper limit checks, so only use that define to alter this value.
	 */
	u16				sequence;
} __attribute__ ((packed));

/*
 * Each received beacon will be translated into a _scan_entry structure
 * from which it can be eaither added to the scan list and if
 * the beacon came from the BSSID we are connected with we can update our
 * configuration accordingly.
 */
struct _scan_entry {
	struct list_head		list;
	unsigned long			timestamp;
	char				essid[IW_ESSID_MAX_SIZE];
	char				bssid[ETH_ALEN];
	s8				rssi;
	u8				mode;
	u8				rates[MAX_BIT_RATES];
	u16				channel;
	u8				atim_count;
	u8				cfp_count;
	u16				atim_period;
	u16				cfp_period;
	u16				beacon;
	u16				capability;
} __attribute__ ((packed));

struct _rt2x00_scan {
	/*
	 * Timstamp from last scan.
	 */
	unsigned long			timestamp;
	/*
	 * Search type. i.e. Only this ESSID only this mode etc.
	 */
	u32				flags;
	/*
	 * List of search results.
	 */
	struct list_head		result;
} __attribute__ ((packed));

struct _dma_ring {
	/*
	 * Address of the descriptor memory.
	 */
	void				*desc_addr;
	/*
	 * Address of the packet memory.
	 */
	void				*data_addr;
} __attribute__ ((packed));

/*
 * This structure is initialized by the rt2x00_pci and rt2x00_usb modules,
 * when they allocate the required memory.
 * They then pass this structure to the main module where we are using the
 * addresses as start addresses for our packet ring arrays.
 */
struct _device_mem {
	struct _dma_ring		rx_ring;
	struct _dma_ring		tx_ring;
	struct _dma_ring		atim_ring;
	struct _dma_ring		prio_ring;
	struct _dma_ring		beacon_ring;

	dma_addr_t			rx_desc_dma;
	dma_addr_t			rx_data_dma;

	dma_addr_t			tx_desc_dma;
	dma_addr_t			tx_data_dma;

	dma_addr_t			atim_desc_dma;
	dma_addr_t			atim_data_dma;

	dma_addr_t			prio_desc_dma;
	dma_addr_t			prio_data_dma;

	dma_addr_t			beacon_desc_dma;
	dma_addr_t			beacon_data_dma;
} __attribute__ ((packed));

struct _rt2x00_dma {
	struct _dma_ring		rx_ring[RX_RING_SIZE];
	struct _dma_ring		tx_ring[TX_RING_SIZE];
	struct _dma_ring		atim_ring[ATIM_RING_SIZE];
	struct _dma_ring		prio_ring[PRIO_RING_SIZE];
	struct _dma_ring		beacon_ring;

	u8				rx_ring_index;
	u8				tx_ring_index;
	u8				atim_ring_index;
	u8				prio_ring_index;

	u8				rx_ring_done_index;
	u8				tx_ring_done_index;
	u8				atim_ring_done_index;
	u8				prio_ring_done_index;
} __attribute__ ((packed));

struct _mac_filter {
	struct list_head		list;
	char				addr[ETH_ALEN];
} __attribute__ ((packed));

struct _duplicate_cache {
	u16				sequence;
	char				addr[ETH_ALEN];
} __attribute__ ((packed));

struct _rt2x00_adapter {
	/*
	 * All spinlocks should be acquired in the order they are defined.
	 */
	spinlock_t			adapter_lock;
	rwlock_t			config_lock;
	spinlock_t			rx_lock;
	spinlock_t			tx_lock;
	spinlock_t			atim_lock;
	spinlock_t			prio_lock;
	spinlock_t			beacon_lock;

	/*
	 * To determine the capabilities of the adapter the exact adapter type should be determined.
	 * chip_adapter - chipset on adapter.
	 * This value is acquired through the driver_data value in the pci_device_id structure.
	 * chip_rf_type - The RF type of the chipset. This value is important when using the RF register.
	 * Each RF type requires a different RF register value. This value is acquired by reading the EEPROM.
	 */
	u16				chip_adapter;
	u16				chip_rf_type;

	/*
	 * Protected by config_lock.
	 */
	u32				flags;

	/*
	 * Protected by config_lock.
	 *
	 * This structure contains the adapter capabilities for the bit rates, channels and tx power.
	 * This depends depends on the chip identification for recognizing capabilities.
	 */
	struct _rt2x00_capability	*capability;

	/*
	 * PCI MMIO Base Address, will be read by the functions:
	 * REGISTER_READ() and REGISTER_WRITE().
	 */
	void __iomem			*register_base_address;

	/*
	 * The CSR8 value (irq mask) is stored here when rt2x00_disable_interrupt() is called.
	 * This value will be written back to the register by rt2x00_enable_interrupt().
	 */
	u32				irq_mask;

	/*
	 * Protected by config_lock.
	 */
	struct _rt2x00_config		*config;

	/*
	 * List of results from the last scan.
	 */
	struct _rt2x00_scan		scan;

	/*
	 * Timer for various opertations.
	 */
	struct timer_list		scan_timer;
	struct timer_list		periodic;

	/*
	 * Protected by config_lock.
	 *
	 * The BSSID which we are connected to.
	 * When we are connecting to an entry in the scan list the entry will be copied,
	 * this means that the scan list can be safely purged without loosing the connection.
	 */
	struct _scan_entry		*connection;

	/*
	 * The dma memory is only used at init and de-init of the device specific modules.
	 */
	struct _device_mem		*dma_mem;

	/*
	 * The DMA memory handlers. The pointers to the mapped DMA memory are stored in here.
	 * All access to the DMA memory should go through here.
	 */
	struct _rt2x00_dma		*dma;

	/*
	 * The tx and prio packet queues are filled by packets that are pending transmission.
	 * These enqueued packets are already completely prepared and only need a sequence number.
	 * All other processing should be done prior the enqueue command.
	 * The skb->cb[48] array should contain data required for the packet descriptors.
	 * The rx packet queue contains the received packets that are not yet filtered or processed.
	 * The skb->cb[48] array contains data from the packet descriptor.
	 * The packet queues are protected by the matching *_lock.
	 * frag_packet_queue is protected by the rx_lock.
	 */
	struct sk_buff_head		rx_packet_queue;
	struct sk_buff_head		tx_packet_queue;
	struct sk_buff_head		prio_packet_queue;
	struct sk_buff_head		frag_packet_queue;

	/*
	 * We use these bottom halves to enqueue the packet handlers to some other time.
	 * We do not need to handle the copy tx and prio packets that are enqueued in the tx_buffer
	 * to the DMA buffer during the interrupt. Nor do we need to perform the packet filtering
	 * of received packets during interrupt. In both cases it is safe to defer the work to later.
	 */
	struct tasklet_struct		tx_tasklet;
	struct tasklet_struct		prio_tasklet;
	struct tasklet_struct		rx_tasklet;

	/*
	 * By using the private IOCTL calls we can add MAC addresses that are allowed on the network.
	 * This filter will only be used in AD-HOC or MASTER mode, since in these 2 modes
	 * it would be usefull to allow only a selective number of addresses with security in mind.
	 */
	struct list_head		*mac_filter;

	/*
	 * When receiving packets we store the source address and the sequence number here,
	 * after we have received another packet we check if the packet is a duplicate
	 * by comparing source address and sequence number.
	 */
	struct _duplicate_cache		duplicate_cache[MAX_DUPLICATE_CACHE];

	/*
	 * Various structures required for network devices,
	 * including spy reports, and network statistics.
	 */
	struct net_device		*net_dev;
	struct iw_spy_data		spy_data;
	struct iw_statistics		iw_stats;
	struct net_device_stats		stats;
} __attribute__ ((packed));

/*
 * Wireless ioctl structure, Initialized in rt2x00_ioctl.c.
 * A direct reference is required at initialization of the netdevice.
 */
extern const struct iw_handler_def rt2x00_ioctl_handler_def;

/*
 * Include the register header here
 * rt2x00_register requires the definition of the _rt2x00_adapter for the register handlers.
 * And in the remainder of this header we require the definitions of the register handlers.
 */
#include "rt2x00_register.h"

/*
 * Wrapper for the 'char cb[40]' variable located in the sk_buff.
 * This structure may not exceed the length of this variable.
 * So make sure length does not exceed 40 bytes (11 words), and 4-byte align all variables.
 * Because of the size limitation in rt2x00_alpha i have split rx and tx and kept common flags
 * it seems to work ok so far
 */
  
struct _skb_cb_rx{
	/* word 1. */
	u32			rts:8;
	u32			fragments:8;
	u32			repeater_packet:8;
	u32			header_length:8;

	/* word 2. */
	u32			seq_ctl:16;
	u32			multicast:8;
	u32			reserved:8;

	/* word 3 - 5. */
	struct _rxd		rxd;
} __attribute__ ((packed));

struct _skb_cb_tx{
	/* word 1. */
	u32			rts:8;
	u32			fragments:8;
	u32			repeater_packet:8;
	u32			header_length:8;

	/* word 2. */
	u32			seq_ctl:16;
	u32			multicast:8;
	u32			reserved:8;

	/* word 3 - 9. */
	struct _txd		txd;

} __attribute__ ((packed));


/*
 * rt2x00_enable_ / rt2x00_disable_interrupt()
 *
 * These 2 functions must be called with adapter->lock held!
 *
 * Before disabling interrupts we store the original mask in adapter->irq_mask.
 * Thos prevents the mask value to be lost when the interrupts are being disabled,
 * and will make it possible that the mask is changed while interrupts remain disabled.
 * When making changes to the irq_mask you should always change adapter->irq_mask
 * and when you want to make the change to take effect immediatly run rt2x00_enable_interrupt().
 * This may even be done when interrupts are enabled, since it will just upload
 * the mask into the register.
 * In rt2x00_disable_interrupt() it does check if the interrupts are already disabled,
 * and in that case it will return immediatly. When disabling the interrupts twice it will
 * write the interrupts_off mask to adapter->irq_mask which means it will not be able to 
 * enable the interrupts again without resetting the device.
 */

static inline void rt2x00_enable_interrupt(struct _rt2x00_adapter* adapter)
{
	rt2x00_register_write(adapter, CSR8, adapter->irq_mask);
	SET_FLAG(adapter, ADAPTER_INTERRUPT_ENABLED);
}

static inline void rt2x00_disable_interrupt(struct _rt2x00_adapter* adapter)
{
	if(unlikely(!TEST_FLAG(adapter, ADAPTER_INTERRUPT_ENABLED)))
		return;
	rt2x00_register_read(adapter, CSR8, &adapter->irq_mask);
	rt2x00_register_write(adapter, CSR8, 0xffffffff);
	CLEAR_FLAG(adapter, ADAPTER_INTERRUPT_ENABLED);
}

/*
 * __connection is the operation mode, i.e. IW_MODE_ADHOC, IW_MODE_INFRA etc...
 */
#define LINK_IS_UP(__adapter)			( (__adapter)->config->active_mode )
#define LINK_TYPE(__adapter, __connection)	( (__adapter)->config->active_mode == (__connection) )
#define LINK_CFG_TYPE(__adapter, __connection)	( (__adapter)->config->config_mode == (__connection) )

/*
 * Link quality.
 */
#define LINK_QUALITY_GOOD(__adapter)		( (__adapter)->connection && ((__adapter)->connection->rssi > LOW_RSSI) )
#define LINK_QUALITY_BAD(__adapter)		( (__adapter)->connection && ((__adapter)->connection->rssi < GOOD_RSSI) )

/*
 * Beacon expiration check.
 */
#define BEACON_IS_LOST(__node)			( time_after(jiffies, ((__node)->timestamp + BEACON_EXPIRE)) )
#define SCAN_IS_EXPIRED(__adapter)		( time_after(jiffies, ((__adapter)->scan.timestamp + BEACON_EXPIRE)) )

/*
 * LED Control
 *	- association: LED 1 - on
 *	- TX-activity: LED 2 - on
 *	- No association: LED 1 & 2 - off
 */
#define LED_ON(__adapter)			do{ rt2x00_register_write((__adapter), LEDCSR, 0x0003461E); } while(0)
#define LED_OFF(__adapter)			do{ rt2x00_register_write((__adapter), LEDCSR, 0x0000461E); } while(0)

#define TXPOWER(__adapter)			( (__adapter)->capability->tx_powers[(__adapter)->config->tx_power] )
#define _TXPOWER(__adapter, __txpower)		( (__adapter)->capability->tx_powers[(__txpower)] )
#define BITRATE(__adapter)			( (__adapter)->capability->bit_rates[(__adapter)->config->cur_rate].value )
#define _BITRATE(__adapter, __rate)		( (__adapter)->capability->bit_rates[(__rate)].value )
#define _BITPLCP(__adapter, __rate)		( (__adapter)->capability->bit_rates[(__rate)].plcp )
#define CHANNEL(__adapter)			( (__adapter)->capability->channels[(__adapter)->config->channel].value )
#define _CHANNEL(__adapter, __channel)		( (__adapter)->capability->channels[(__channel)].value )
#define CHANNELRF(__adapter)			( (__adapter)->capability->channels[(__adapter)->config->channel].rf )
#define _CHANNELFREQ(__adapter, __channel)	( (__adapter)->capability->channels[(__channel)].freq )
#define BASIC_RATE(__adapter)			( (__adapter)->capability->basic_rate )

/* 
 * Duration calculations
 *  For these functions we expect the rate to be passed as the value stored in the adapter capability structure.
 *  This means the rate_value is: 2 * real_rate (in Mb/s).
 *  Therefore length has to be multiplied with 8 to convert to bits as well as mulltiply the length
 *  with 2 to compensate for the difference between real_rate and the rate_value.
 *
 * Usage :
 *  u16 DURATION(u16 size, u8 rate)
 *  u16 DURATION_RES(u16 size, u8 rate)
 */
#define DURATION(__size, __rate)		( (u16)(((__size) * 8 * 2) / (__rate)) )
#define DURATION_RES(__size, __rate)		( (u16)(((__size) * 8 * 2) % (__rate)) )

static inline u16 rt2x00_packet_duration(struct _rt2x00_adapter *adapter, u8 rate, u16 size)
{
	return adapter->config->plcp + adapter->config->preamble + DURATION(size, adapter->capability->bit_rates[rate].value);
}

/*
 * A multicast address starts with 01:XX:XX:XX:XX:XX.
 */
#define ADDR_IS_MULTICAST(__addr)		( (__addr)[0] & 0x01 )

/*
 * Altough the sequence value is an unsigned 16bit type, the first 4 bits are not part of the sequence number.
 * Since we don't want to shift the value to the correct bits each time we use the value,
 * we increase the sequence with 0x0010 and make sure that when the max is reached the value is set back to 0.
 */
#define FRAME_SEQ_INC(__adapter) \
	do{ (__adapter)->config->sequence = ((__adapter)->config->sequence + 0x0010) & (IEEE802_11_SCTL_SEQ); }while(0)

/*
 * Increase the ring index, when the max value is reached switch back to 0.
 */
#define INCREASE_RING(__index, __max)		do{ (__index) = ((++(__index) < (__max)) ? (__index) : 0); }while(0)

/*
 * Byte ordering
 * All data coming from, and going to the device should have the correct byte order.
 * The kernel provides le32_to_cpu and cpu_to_le32 and these functions require 32bit data.
 * Since we need to convert larger buffers, we must cut the buffer in smaller (32 bits) pieces
 * and hand each piece to the kernel for byte ordering.
 * Make sure that at every copy we check the length of the remaining data.
 */
static inline void device_to_cpu(void *dest, void *data, int length)
{
	u32	*part = (u32*)data;
	u32	buffer;

	while(length > 0){
		buffer = le32_to_cpu(*part);

		memcpy(dest, &buffer, (length >= sizeof(u32)) ? sizeof(u32) : length);

		dest += sizeof(u32);
		part ++; /* part is a u32 pointer*/
		length -= sizeof(u32);
	}
}

static inline void cpu_to_device(void *dest, void *data, int length)
{
	u32	*part = (u32*)data;
	u32	buffer;

	while(length > 0){
		buffer = le32_to_cpu(*part);

		memcpy(dest, &buffer, (length >= sizeof(u32)) ? sizeof(u32) : length);

		dest += sizeof(u32);
		part ++; /* part is a u32 pointer*/
		length -= sizeof(u32);
	}
}

/*
 * The following functions are declared extern because they are to be used in the device specific modules.
 */
extern struct net_device* rt2x00_probe(u16 chip_type, void __iomem *csr_addr, struct _device_mem *dma);
extern void rt2x00_remove(struct net_device *net_dev);
extern irqreturn_t rt2x00_interrupt(int irq, void *dev_instance, struct pt_regs *regs);
extern int rt2x00_suspend(struct _rt2x00_adapter *adapter, u32 state);
extern int rt2x00_resume(struct _rt2x00_adapter *adapter);

/*
 * Adapter main functions.
 */
void rt2x00_load_config(struct _rt2x00_adapter* adapter);
void rt2x00_update_adapter(struct _rt2x00_adapter* adapter);

/*
 * Packet handler functions.
 */
char rt2x00_enqueue_packet(struct _rt2x00_adapter *adapter, struct sk_buff *skb);

/*
 * Initialization functions.
 */
char rt2x00_init_adapter_eeprom(struct _rt2x00_adapter *adapter);
char rt2x00_init_adapter_capable(struct _rt2x00_adapter *adapter);
char rt2x00_init_adapter_config(struct _rt2x00_adapter *adapter);
char rt2x00_init_adapter_register(struct _rt2x00_adapter *adapter);
char rt2x00_init_schedule_irq(struct _rt2x00_adapter *adapter);
char rt2x00_init_dma_ring(struct _rt2x00_adapter *adapter);

/*
 * Scan functions.
 */
void rt2x00_start_scan(struct _rt2x00_adapter *adapter, u8 flags);
void rt2x00_end_scan(struct _rt2x00_adapter *adapter);
struct _scan_entry *rt2x00_search_scan_entry(struct _rt2x00_adapter *adapter, u8 search_mode);
void rt2x00_update_scan_entry(struct _rt2x00_adapter *adapter, struct _scan_entry *scan_entry);

/*
 * Management functions.
 */
void rt2x00_start_beacon(struct _rt2x00_adapter *adapter, u8 mode);
void rt2x00_halt_beacon(struct _rt2x00_adapter *adapter);
void rt2x00_send_ps_poll(struct _rt2x00_adapter *adapter);
void rt2x00_send_null_frame(struct _rt2x00_adapter *adapter);
char rt2x00_transmit_mgmt(struct _rt2x00_adapter *adapter, struct sk_buff *skb);
char rt2x00_receive_mgmt_packet(struct _rt2x00_adapter *adapter, struct sk_buff *skb);

/*
 * DMA memory control
 */
void rt2x00_cleanup_dma_ring(struct _rt2x00_adapter *adapter, u8 ring_type);

/*
 * interrupt functions.
 */
u8 rt2x00_test_free_descriptors(struct _rt2x00_adapter *adapter, u8 ring, u8 fragments);
void rt2x00_write_tx_descriptor(struct _rt2x00_adapter *adapter, struct _txd *txd, u8 *desc, u32 rate, u32 length);
void rt2x00_interrupt_tcbn_expire(struct _rt2x00_adapter *adapter);
void rt2x00_interrupt_twake_expire(struct _rt2x00_adapter *adapter);
void rt2x00_interrupt_tatimw_expire(struct _rt2x00_adapter *adapter);
void rt2x00_interrupt_txdone_txring(struct _rt2x00_adapter *adapter);
void rt2x00_interrupt_txdone_atimring(struct _rt2x00_adapter *adapter);
void rt2x00_interrupt_txdone_prioring(struct _rt2x00_adapter *adapter);
void rt2x00_interrupt_rxdone(struct _rt2x00_adapter *adapter);

/*
 * Tasklets functions.
 */
void rt2x00_delayed_start_tx(unsigned long data);
void rt2x00_delayed_start_tx_prio(unsigned long data);
void rt2x00_delayed_handle_rx(unsigned long data);

/* Connection functions. */
void rt2x00_radio_on(struct _rt2x00_adapter* adapter);
void rt2x00_radio_off(struct _rt2x00_adapter* adapter);
void rt2x00_link_up(struct _rt2x00_adapter *adapter);
void rt2x00_link_down(struct _rt2x00_adapter *adapter);

/*
 * Encryption functions.
 */
char rt2x00_init_key(struct _rt2x00_adapter *adapter);
char rt2x00_del_key(struct _rt2x00_adapter *adapter, u8 index);
char rt2x00_encrypt(struct _rt2x00_adapter *adapter, struct sk_buff *skb);
char rt2x00_decrypt(struct _rt2x00_adapter *adapter, struct sk_buff *skb);
char rt2x00_add_key(struct _rt2x00_adapter *adapter, struct _crypto *add_key);
char rt2x00_get_key(struct _rt2x00_adapter *adapter, struct _crypto *get_key);

#endif /* __RT2X00_ADAPTER_H */
