/*
	rt2x00 SourceForge Project - <http://rt2x00.serialmonkey.com>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the
	Free Software Foundation, Inc.,
	59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

	Licensed under the GNU GPL
	Original code supplied under license from RaLink Inc, 2003
 */

/*
	Module Name: rt2x00_defines.h

	Abstract: General used defines.

	Revision History:
	Who		When		What
	--------	-----------	-----------------------------
	IvD		<releasedate>	Initial Release.
 */

#ifndef __RT2X00_DEFINES_H
#define __RT2X00_DEFINES_H

#define ACK_SIZE		14

#define MAX_BIT_RATES		12
#define MAX_FREQUENCIES		38

#define MAX_DUPLICATE_CACHE	4

#define MAX_REGISTER_BUSY_COUNT	10	/* Number of retry before failing access BBP & RF indirect register */
#define REGISTER_BUSY_DELAY	100	/* Delay between each bbp_read retry. (us) */

#define CRYPTO_SIZE_WEP_SMALL	5	/* 40 bits WEP */
#define CRYPTO_SIZE_WEP_LARGE	13	/* 104 bits WEP */
#define CRYPTO_SIZE_WPA		32	/* 256 bits WPA */

#define TX_EXPIRE		HZ

#define MIN_FRAGMENTATION	256	/* byte count */
#define MAX_FRAGMENTATION	2312	/* byte count */
#define MIN_RTS_THRESHOLD	0	/* byte count */
#define MAX_RTS_THRESHOLD	2312	/* byte count */

#define HIGH_RETRY_ERROR_FRAG	40	/* Percentage of packets send with retry. */
#define LOW_RETRY_ERROR_FRAG	10	/* Percentage of packets send with retry. */
#define HIGH_RETRY_ERROR_RTS	60	/* Percentage of packets send with retry. */
#define LOW_RETRY_ERROR_RTS	40	/* Percentage of packets send with retry. */

#define SENSITIVITY_DEFAULT	-75

#define MAX_RSSI		-20	/* dBm */
#define MIN_RSSI		-100	/* dBm */
#define LOW_RSSI		-50	/* dBm */
#define GOOD_RSSI		-25	/* dBm */

#define MIN_POWER_PERIOD	1024
#define MAX_POWER_PERIOD	65535
#define MIN_POWER_TIMEOUT	1
#define MAX_POWER_TIMEOUT	1000

#define PREAMBLE_LONG_DURATION	144
#define PREAMBLE_SHORT_DURATION	72

/* DMA ring definitions */
#define RX_RING			0x01
#define TX_RING			0x02
#define ATIM_RING		0x04
#define PRIO_RING		0x08
#define BEACON_RING		0x10

#define RX_RING_SIZE		8
#define TX_RING_SIZE		4
#define ATIM_RING_SIZE		2
#define PRIO_RING_SIZE		2
#define BEACON_RING_SIZE	1

#define RX_BUFFER_SIZE		2048
#define TX_BUFFER_SIZE		2048
#define ATIM_BUFFER_SIZE	2048
#define PRIO_BUFFER_SIZE	1024
#define BEACON_BUFFER_SIZE	2048

#define RING_DESCRIPTOR_SIZE	48

#define RX_DATA_SIZE		( RX_RING_SIZE * RX_BUFFER_SIZE )
#define TX_DATA_SIZE		( TX_RING_SIZE * TX_BUFFER_SIZE )
#define ATIM_DATA_SIZE		( ATIM_RING_SIZE * ATIM_BUFFER_SIZE )
#define PRIO_DATA_SIZE		( PRIO_RING_SIZE * PRIO_BUFFER_SIZE )
#define BEACON_DATA_SIZE	( BEACON_RING_SIZE * BEACON_BUFFER_SIZE )

#define RX_DESC_SIZE		( RX_RING_SIZE * RING_DESCRIPTOR_SIZE )
#define TX_DESC_SIZE		( TX_RING_SIZE * RING_DESCRIPTOR_SIZE )
#define ATIM_DESC_SIZE		( ATIM_RING_SIZE * RING_DESCRIPTOR_SIZE )
#define PRIO_DESC_SIZE		( PRIO_RING_SIZE * RING_DESCRIPTOR_SIZE )
#define BEACON_DESC_SIZE	( BEACON_RING_SIZE * RING_DESCRIPTOR_SIZE )

#define MIN_PACKET_SIZE		256
#define MAX_PACKET_SIZE		TX_BUFFER_SIZE

#define DESC_OWNED_BY_NIC	1
#define DESC_OWNED_BY_HOST	0

#define IFS_BACKOFF		0x00
#define IFS_SIFS		0x01

#define SCAN_TIME		( 1 * HZ )	/* seconds */
#define PERIODIC_EXEC		( HZ )		/* seconds */
#define BEACON_EXPIRE		( 2 * HZ )	/* seconds */

#define TX_SUCCESS		0
#define TX_SUCCESS_RETRY	1
#define TX_FAIL_RETRY		2
#define TX_FAIL_INVALID		3
#define TX_FAIL_OTHER		4

#endif /* __RT2X00_DEFINES_H */
