/*-
 * Copyright (c) 2005
 *	Damien Bergamini <damien.bergamini@free.fr>
 * Copyright (c) 2005
 *	Jeremy Bobbio <jeremy.bobbio@etu.upmc.fr>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#define RAL_SKB_POOL_SIZE	10

struct ural_rx_radiotap_header {
	struct ieee80211_radiotap_header wr_ihdr;
	u_int8_t		wr_flags;
	u_int16_t		wr_chan_freq;
	u_int16_t		wr_chan_flags;
	u_int8_t		wr_antenna;
	u_int8_t		wr_antsignal;
};

#define RAL_RX_RADIOTAP_PRESENT						\
	((1 << IEEE80211_RADIOTAP_FLAGS) |				\
	 (1 << IEEE80211_RADIOTAP_CHANNEL) |				\
	 (1 << IEEE80211_RADIOTAP_ANTENNA) |				\
	 (1 << IEEE80211_RADIOTAP_DB_ANTSIGNAL))

struct ural_tx_radiotap_header {
	struct ieee80211_radiotap_header wt_ihdr;
	u_int8_t		wt_flags;
	u_int8_t		wt_rate;
	u_int16_t		wt_chan_freq;
	u_int16_t		wt_chan_flags;
	u_int8_t		wt_antenna;
};

#define RAL_TX_RADIOTAP_PRESENT						\
	((1 << IEEE80211_RADIOTAP_FLAGS) |				\
	 (1 << IEEE80211_RADIOTAP_RATE) |				\
	 (1 << IEEE80211_RADIOTAP_CHANNEL) |				\
	 (1 << IEEE80211_RADIOTAP_ANTENNA))

struct ural_softc;

struct ural_softc {
	struct net_device		*netdev; /* Must be first */
	struct usb_device		*udev;
	struct net_device_stats		sc_devstats;
	struct iw_statistics		sc_iwstats;
	struct kref			kref;

	uint8_t				sc_flags;
#define RAL_RX_URB_FAILED		(1 << 0)
#define RAL_UNPLUG			(1 << 1)
#define RAL_MONITOR_SKB			(1 << 2)

	struct ieee80211com		sc_ic;
	int				(*sc_newstate)(struct ieee80211com *,
					    enum ieee80211_state, int);
#if 0
	XXX: for reference only, to remove
	USBBASEDEVICE			sc_dev;
	usbd_device_handle		sc_udev;
	usbd_interface_handle		sc_iface;
#endif

	u_int32_t			asic_rev;
	u_int8_t			rf_rev;

	unsigned int			sc_rx_pipe;
	u_int8_t			sc_rx_no;
	struct urb			*sc_rx_urb;
	int				sc_rx_buf_size;
	struct sk_buff			*sc_rx_skb;
	struct sk_buff			*rx_skb_pool[RAL_SKB_POOL_SIZE];
	spinlock_t			rx_pool_lock;

	unsigned int			sc_tx_pipe;
	u_int8_t			sc_tx_no;
	struct urb			*sc_tx_urb;
	struct sk_buff			*sc_tx_skb;
	u_int8_t			*sc_tx_buf;
	struct ieee80211_node		*sc_tx_ni;

	enum ieee80211_state		sc_state;
	
	struct work_struct		sc_work;
	struct work_struct		sc_scan_ch;
	struct work_struct		sc_init;
	struct work_struct		sc_mode_init;

	struct tasklet_struct		rx_fixup_task;

	struct ieee80211_beacon_offsets	sc_bo;

	struct semaphore		sc_lock;

	u_int32_t			rf_regs[4];
	u_int8_t			txpow[14];

	struct {
		u_int8_t		val;
		u_int8_t		reg;
	} __attribute__((__packed__))	bbp_prom[16];

	int				led_mode;
	int				hw_radio;
	int				rx_ant;
	int				tx_ant;
	int				nb_ant;
};

#define RAL_LOCK_INIT(_sc)	init_MUTEX(&(_sc)->sc_lock)
#define RAL_LOCK_DESTROY(_sc)	
#define RAL_LOCK(_sc)		down(&(_sc)->sc_lock)
#define RAL_UNLOCK(_sc)		up(&(_sc)->sc_lock)

/*
 * For packet capture, define the same physical layer packet header 
 * structure as used in the wlan-ng driver 
 */
enum {
	DIDmsg_lnxind_wlansniffrm		= 0x00000044,
	DIDmsg_lnxind_wlansniffrm_hosttime	= 0x00010044,
	DIDmsg_lnxind_wlansniffrm_mactime	= 0x00020044,
	DIDmsg_lnxind_wlansniffrm_channel	= 0x00030044,
	DIDmsg_lnxind_wlansniffrm_rssi		= 0x00040044,
	DIDmsg_lnxind_wlansniffrm_sq		= 0x00050044,
	DIDmsg_lnxind_wlansniffrm_signal	= 0x00060044,
	DIDmsg_lnxind_wlansniffrm_noise		= 0x00070044,
	DIDmsg_lnxind_wlansniffrm_rate		= 0x00080044,
	DIDmsg_lnxind_wlansniffrm_istx		= 0x00090044,
	DIDmsg_lnxind_wlansniffrm_frmlen	= 0x000A0044
};
enum {
	P80211ENUM_msgitem_status_no_value	= 0x00
};
enum {
	P80211ENUM_truth_false			= 0x00
};

typedef struct {
	u_int32_t did;
	u_int16_t status;
	u_int16_t len;
	u_int32_t data;
} p80211item_uint32_t;

typedef struct {
	u_int32_t msgcode;
	u_int32_t msglen;
#define WLAN_DEVNAMELEN_MAX 16
	u_int8_t devname[WLAN_DEVNAMELEN_MAX];
	p80211item_uint32_t hosttime;
	p80211item_uint32_t mactime;
	p80211item_uint32_t channel;
	p80211item_uint32_t rssi;
	p80211item_uint32_t sq;
	p80211item_uint32_t signal;
	p80211item_uint32_t noise;
	p80211item_uint32_t rate;
	p80211item_uint32_t istx;
	p80211item_uint32_t frmlen;
} wlan_ng_prism2_header;

/* vim: noet sw=8
 */
